use serial::{unix::TTYPort, BaudRate::BaudOther, SerialPort};
use std::{
    io::{Read, Write},
    time::Duration,
};

use crate::packet::FecPacket;

// Used for talking to an RF4463 via a microcontroller
// The microcontroller takes in packet data over UART and sends it to the RF4463
// Note that this is a little hacky. It's only intended to be used
// for debugging, and not for an actual balloon flight
pub struct UartRadio {
    uart: TTYPort,
}

impl UartRadio {
    pub fn new(uart: &str) -> Self {
        let mut uart = serial::open(uart).unwrap();
        uart.reconfigure(&|settings| settings.set_baud_rate(BaudOther(921600)))
            .unwrap();
        uart.set_timeout(Duration::from_millis(5)).unwrap();

        Self { uart }
    }

    pub fn send_packet(&mut self, packet: &FecPacket) {
        self.uart.write_all(&packet.0).unwrap();
        let mut buf = [0; 1];
        if let Ok(1) = self.uart.read(&mut buf) {
            if buf[0] == b'F' {
                // uC is full. Need to wait until we receive
                // the empty signal
                while buf[0] != b'E' {
                    let _ = self.uart.read(&mut buf);
                }
            }
        }
    }
}
